#
# Copyright 2010, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Josef Skladanka <jskladan@redhat.com>

from autotest_lib.client.bin import test, utils
from autotest_lib.client.bin.test_config import config_loader

from decorators import ExceptionCatcher
from util import make_autotest_url
from config import getbool
import os, sys, traceback

class AutoQATest(test.test, object):
    version = 1 # increment if setup() changes

    def __init__(self, *args, **kwargs):
        super(AutoQATest, self).__init__(*args, **kwargs)
        self.result = None #overall test result ~ "PASSED", "FAILED"...
        self.summary = "" #used as mail-subject
        self.highlights = "" #used as first part of mail-body
        self.outputs = "" #second part of mail_body
        self.mail_to = [] #list of emails to send results to. Mail is always sent to results list

    @ExceptionCatcher()
    def setup(self, **kwargs):
        pass

    @ExceptionCatcher()
    def initialize(self, config, **kwargs):
        self.config = config_loader(config, self.tmpdir)
        self.autotest_url = make_autotest_url(self.config)

    @ExceptionCatcher()
    def run_once(self, **kwargs):
        os.chdir(self.bindir) # easiest way for tests to find their test scripts, config files, etc

    def process_exception(self, exc):
        self._convert_list_variables()

        if self.result == 'ABORTED':
            # the exception is raised intentionally, don't override anything,
            # only if empty
            if not self.summary:
                self.summary = "%s: %s" % (exc[1].__class__.__name__, exc[1])
        else:
            self.result = "CRASHED"
            self.summary = "%s: %s" % (exc[1].__class__.__name__, exc[1])

        # append traceback
        self.outputs += '\n%s\n%s' % ('-'*70,
            ''.join(traceback.format_exception(exc[0], exc[1], exc[2])))

        try:
            self.postprocess_iteration()
        except Exception:
            traceback.print_exc(file=sys.stderr)

    def postprocess_iteration(self):
        self._convert_list_variables()

        if self.result is None:
            self.result = "NEEDS_INSPECTION"
        if self.summary is None or len(self.summary) == 0:
            self.summary = ""
        self.summary = "%s: %s; %s" % (self.__class__.__name__, self.result, self.summary)

        # prepare email
        send_to = []
        result_email = self.config.get('notifications','result_email')
        if result_email and isinstance(result_email, str):
            send_to = result_email.split(',')
            try:
                for email in self.mail_to:
                    if isinstance(email, str) and len(email) > 0:
                        send_to.append(email)
            except:
                pass
        mail_body = "Stored logs available at <%s>\n\n" % self.autotest_url
        if self.highlights:
            mail_body += "%s\n" % self.highlights
        mail_body += "-"*70 + "\n"
        mail_body += self.outputs

        # print results to the screen
        summary_table = '\n'.join([
        '*'*40,
        '* RESULT: %s' % self.result,
        '* SUMMARY: %s' % self.summary,
        '* HIGHLIGHTS: %d lines' % len(self.highlights.splitlines() if self.highlights else ''),
        '* OUTPUTS: %d lines' % len(self.outputs.splitlines() if self.outputs else ''),
        '* EMAIL RECIPIENTS: %s' % ', '.join(send_to),
        '*'*40])
        print summary_table

        # store the whole output in a log file
        log = open(os.path.join(self.resultsdir,'output.log'),'wb')
        log.write(summary_table)
        log.write('\n\n')
        log.write(mail_body)
        log.close()

        # send email
        if send_to:
            utils.send_email(mail_to=send_to,
                mail_from=self.config.get('notifications','mail_from'),
                mail_server=self.config.get('notifications','smtpserver'),
                subject=self.summary,
                body = mail_body)

    def _convert_list_variables(self):
        '''
        We allow some variables to be as lists. We now need them as strings,
        so let's just join them with newlines.
        '''
        if isinstance(self.outputs, list):
            self.outputs = '\n'.join(self.outputs)
        if isinstance(self.highlights, list):
            self.highlights = '\n'.join(self.highlights)

